.comment All CBUGS entries have the following format:
.comment
.comment	.tp 4.s 2
.comment	.br <semicolon>**
.comment	.br <semicolon>**##^&Title of the problem\&
.comment	.br <semicolon>**
.comment	.s
.comment	[[ problem text ]]
.comment
.comment CBUGS.DOC is build using the following RNO command:
.comment
.comment	RNO CBUGS.DOC=[5,1]CBUGS.RNO/UL:N
.comment
.comment This supresses underlines and starts entries at column 1.
.comment To make CC.DOC, append CBUGS.RNO to CC.RNO before executing
.comment RNO.TSK
.comment
.ps 58,80.rm 72.nhy
.f
.s
This document contains the current list of errors and restrictions
in the DECUS C language system.  As errors are removed, they will
be deleted from this file.  Anything in this file should be regarded
as permanent.  The error list is in no particular order.
.tp 4.s 2
.br ;**
.br ;**##^&Long input lines crash the compiler\&
.br ;**
.s
If the source file contains a long input line, the compiler will
crash with an "abort in phase 0" message.  One source of compiler
aborts is an input line that becomes very long when "_#define" statements
are expanded.  "long" in this context means 132 bytes.
.tp 4.s 2
.br ;**
.br ;**##^&Bad syntax may crash the compiler\&
.br ;**
.s
The construction
.s
	return((c < 0) -1 : 0);
.s
(with a missing "?") aborts in phase 2 with a "missing code table" error.
It should yield a syntax error message.
.s
A similar problem occurs when the programmer writes ':' when ';' was
intended:
.s.nf
	printf("foo_\n"):	/* Note colon	*/
	printf("bar_\n");
.s
or
	c = 'U':		/* Note colon	*/
	pointer--;
.s.f
Gives a confusing "Abort in pass 2" error message ("Missing
code table entry for CLN").  The problem is that the syntax analyser
does not parse ternary expressions ("a = (b > c) ? d : e") correctly;
accepting the colon without a preceeding '?'.  The expression tree
which is subsequently built cannot be processed by the code generator.
.s
Since symbols are of finite length, there can be cases where a program
compiles correctly, but fails in the assembler (AS) module.  For example,
.s.nf
	int	watchrule;
	int	watchrules;
.s.f
Gives a "multiply defined symbol" error in AS, rather than in CC.
.s
The sequence
.s.nf
	int jmp__buf foo[6];
.s
	setjmp(jmp__buf);
.s.f
Compiles without error (it should yield a syntax error), then
crashes the AS assembler.
.tp 4.s 2
.br ;**
.br ;**##^&External functions must be marked extern\&
.br ;**
.s
External functions must be declared as:
.s.nf
	extern int func();
.s
Writing
.s
	int func();
.s.f
may result in a syntax error message.
.tp 4.s 2
.br ;**
.br ;**##^&Long composite constants compile incorrectly\&
.br ;**
.s
The compiler incorrectly computes long constants such as "1<<16".
These must be computed by hand and entered as exact values.
There are two problems here.  One is that the compiler computes the
above expression in "int" precision, assigning a garbage value to
the long constant.  If you "fix" this, by writing "1L<<16", the
compiler will reject the compile-time computation of long expressions,
which is currently unsupported -- see next entry.
.tp 4.s 2
.br ;**
.br ;**##^&Long constant expressions are not optimized\&
.br ;**
.s
If a program contains a constant expression involving longs, such as:
.s
	func(1L + 1L);
.s
the compiler does not generate
.s
	func(2L);
.s
What is more serious is that compiling:
.s
	long value = 1L + 1L;
.s
results in a "bad initializer" error message.
While the fix is straightforward (to routine MODIFY: in CC201.MAC, in
case you're interested), it is a pain in the neck to implement.
.tp 4.s 2
.br ;**
.br ;**##^&Repeated formal parameters\&
.br ;**
.s
If a function definition contains the same formal parameter twice,
as in:
.s
	func(foo, bar, foo)
.s
the compiler does not generate an error message.
.tp 4.s 2
.br ;**
.br ;**##^&Intermediate files written to user's current directory\&
.br ;**
.s
Early versions of RSX CC (from the Decus SIG distribution from
1978) wrote intermediate files and the compiler
output (.s) file on the same disk/directory as the input file.
This has been changed so as to write all output files onto the
user's current directory.  This is compatible with RT11 CC and
general PDP11 practice.  For example, assuming RSTS/E:
.s.nf
	     Command		Old		  New
	xcc [100,100]foo   [100,100]foo.s	sy:foo.s
.s.f
Note that RT11 CC does a "normal" CSI scan, thus allowing placement
of all files.
.tp 4.s 2
.br ;**
.br ;**##^&Error processing relative branches in AS\&
.br ;**
.s
The AS assembler does not process self-relative branches (br .+4)
correctly.  They must not be used.  (Code generated by CC does not
contain relative branches.)
.tp 4.s 2
.br ;**
.br ;**##^&Errors in C parser\&
.br ;**
.s
.s.nf
	if (...)
		do {
			...
		} while (...);
	else
		...
.s.f
is rejected with the message "illegal else".  Rewrite as:
.s.nf
	if (...) {
		do {
			...
		} while (...);
	}
	else ...
.s.f
This makes the syntax explicit for the parser (and programmer).
.s
The parser may incorrectly assign expression type.  For example,
.s.nf
	char	c, *str;
	int	foo, mask;
.s
	foo = mask | (c = *str++);
.s.f
The parser thinks that "(c = *str++)" is of type "pointer" and
cannot compute the inclusive or operation.
.tp 4.s 2
.br ;**
.br ;**##^&Old-style assigned binary operators\&
.br ;**
.s
The C compiler really and truely does not support the old
assigned binary operators (=+, =-, etc.).  In fact, the
statement "foo=-6" is exactly equivalent to "foo = (-6)"; it IS NOT
equivalent to "foo = foo - 6".  The Unix compiler processed the
above ambiguous case by "taking the longest matching lexical entity."
If the statement was written with embedded blanks as "foo#=#-6", the
Unix compiler would, in effect, recognize the blank space between '='
and '-'.
Note that rejecting '=-' is,
an error in Decus C.  This same problem may
be seen in statements such as "foo=*x" which Decus C interprets
as "foo#=#*x" while the Unix (or Vax-11 C) compiler interprets
as "foo#=*#x".
The C syntax definition
.tp 4.s 2
.br ;**
.br ;**##^&Structures must be defined globally\&
.br ;**
.s
The C compiler rejects structure definitions within the body
of a function:
.s.nf
	foobar() {
		struct foostr {
			int *foo;
		} bar;
	}
.s.f
However, if the structure definition is moved outside the function
body, it will compile correctly:
.s.nf
	struct foostr {
		int *foo;
	};

	foo() {
		foostr bar;
	}
.s.f
Note that this means that structures may not be defined one way in
one function and another way in another function.
.tp 4.s 2
.br ;**
.br ;**##^&Random global symbols may prevent task-building\&
.br ;**
.s
The RSX file services emulator library under RSTS/E contains a
global symbol "EOF".  Consequently, C programs running under this
library must not define a global by this name.  The following will
not task-build correctly on RSTS/E, RSX-11 mode:
.s.nf
	int eof;
	main() {
		...
	}
.s.f
Note that this restriction is due to the global's presence in
the operating-system library, not the C run-time library.
.s
In general, note that global symbols that match file services support
globals may cause conflicts with other file services routines.  This
will generally result in undefined global symbols when task building.
For example, functions should not begin with a leading underscore '__'
to avoid matching RSX file-control service routine names.
.s
.tp 4.s 2
.br ;**
.br ;**##^&Structure definitions must be ordered\&
.br ;**
.s
The C compiler is restrictive as to the ordering of definitions.
For example, given the following structure definition:
.s.nf
	struct stack {
		int	max__index;
		int	current__index;
		int	*vector;
	};
.s.f
The compiler rejects the following sequence:
.s.nf
	struct stack	entry = { DATUM__MAX, 0, datum };
	int		datum[DATUM__MAX];
.s.f
(as datum is not yet defined).
However, it accepts the following sequence:
.s.nf
	int		datum[DATUM__MAX];
	struct stack	entry = { DATUM__MAX, 0, datum };
.s.f
.tp 4.s 2
.br ;**
.br ;**##^&Fwild requires a sorted directory on RSX-11M\&
.br ;**
.s
Fwild/fnext will not properly process versions ;0 and ;-1 on
native RSX systems that support FILES-11 (ODS-1) disk directory structures.
The algorithm works correctly on ODS2 structures (and thus on VMS
compatiblity mode).  The problem is that, on ODS-1, files are not
sorted in the directory.  A Decus program,
SRD.TSK, may be used to sort ODS-1 directories
as needed.  SRD is not included on the current Decus C distribution
because of size limitations.
.s
";-1" versions do not seem to work correctly.
.tp 4.s 2
.br ;**
.br ;**##^&Strange RSX-11M file and record formats\&
.br ;**
.s
Several RSX-11M and VAX utility programs generate output files that are
not correctly processed by the C library.  For example, VMS "print-image"
files may encode blank lines in the record sequence field.  As currently
coded, the stream I/O routines (getc(), fgets(), etc.) will drop such lines.
.s
Also, some utility programs write "unformatted" files, with multiple
text lines stored in one (file-system) logical record.  (Task-builder
map files and output from Runoff offer examples of
this phenomenon.)  In this case, sequential file reading is no problem
but, if it is necessary to randomly process the file using ftell()
and fseek(), you should note that the record-address returned by ftell()
refers only to the file-system logical record -- i.e., it will not
distinguish between the various text lines.
.s
The octal-dump program, OD.C (in the tools package), will print record
sequence numbers if they are present.
.s
The file typeout program, T.C (in the tools package), illustrates deblocking
the various file formats and random access to multiple text lines.
.s
Because stdout and stderr are assigned to different channels (luns),
mixing output to stdout and stderr may result in inadvertant overprinting
of output lines.  Prefacing error message output with a newline
'_\n' should prevent this problem.
.s
.tp 4.s 2
.br ;**
.br ;**##^&Library Name Changes\&
.br ;**
.s
In order to bring the library more in line with other C standard I/O
libraries, (primarily the VAX native C compiler), a few library functions
have changed names.  Sorry for the inconvenience.
.s.nf
Old name	Becomes
.s
index		strchr		Search string for character
rindex		strrch		Last instance of char in string
iovtoa		fgetname	Return file name (not on Unix)
sbreak		sbrk		Get incremental memory
fmkdl		(obsolete)	Use delete() in new programs
delete				Delete a file, given its name
ftty		isatty		TRUE if a terminal -- see below
flun		fileno		Logical unit number -- see below
.s.f
The string search functions, strchr and strrch are still index and
rindex on some operating systems, notably 4.2BSD.  On such systems,
you may need to compile programs with the following command:
.s.nf
	cc -Dstrchr=index -Dstrrch=rindex file.c
.s.f
Logical unit numbers used in Decus C bear no relation to Unix file numbers.
In particular, stdin, stdout, and stderr are not 0, 1, and 2.  In fact,
on some operating systems, stderr equals -1.  The proper test for
"is this device a terminal" is now:
.s
	isatty(fileno(fd));
.s
The RSX-11M logical unit number is returned by fileno(fd).
Decus C cannot determine the RSTS/E channel number.
.tp 4.s 2
.br ;**
.br ;**##^&Fseek/Ftell Changes\&
.br ;**
.s
A long-standing error in the RSX versions of
fseek() and ftell() was corrected.
Unfortunately, this means that any data files using cross-file
pointers must be rebuilt from scratch.  (An index file built
using the old version of ftell() cannot be used with a program
linked with the new version of fseek().)
.tp 4.s 2
.br ;**
.br ;**##^&Embedded switch statement restriction\&
.br ;**
.s
Multiple nested switch statements may cause the compiler to
abort with a stack overflow message.  Whether a given construction
is acceptable depends on the overall structure of your program.
.s
The stack size may be changed by recompiling the compiler, but this
lowers the number of symbols that can be accomodated.
.s.f
